/*
 * PolicyApiSample.java
 *
 * Copyright (c) 2007 CA.  All rights reserved.
 *
 * This software and all information contained therein is confidential
 * and proprietary and shall not be duplicated, used, disclosed or
 * disseminated in any way except as authorized by the applicable
 * license agreement, without the express written permission of CA.
 * All authorized reproductions must be marked with this language.
 *
 * EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE
 * EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS SOFTWARE
 * WITHOUT WARRANTY OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT WILL CA BE LIABLE TO THE END USER OR ANY
 * THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR INDIRECT, FROM THE
 * USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST PROFITS,
 * BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS
 * EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
 *
 */

/*
 *
 *              SiteMinder Java Policy API sample program.
 *
 * Usage: java PolicyApiSample [-l] [agent5x]
 *
 *        -l    Logs the output to a file
 *    agent5x Creates a 5.x Agent, otherwise 4.x Agent
 *
 * (see the classpath options in the supplied
 * java-run.bat and java-run.sh scripts)
 */


import java.io.*;
import java.net.*;
import java.util.*;

import com.netegrity.sdk.apiutil.*;
import com.netegrity.sdk.policyapi.*;
import netegrity.siteminder.javaagent.*;

public class PolicyApiSample
{
    // Logging
    private static PrintWriter logWriter    = null;
    private static final String CRLF        = System.getProperty("line.separator");

    // Resource bundle (smjsdksample.properties)
    private static ResourceBundle bundle    = null;
    private static final String BUNDLE_NAME = "smjsdksample";

    // Literal strings to be initialized from the resource bundle
    private static String SAMPLE_DESC       = null;
    private static String OBJNAME_PREFIX    = null;
    private static String STATUS_OK         = null;
    private static String STATUS_NOK        = null;

    // Do we run this sample standalone, or to create the policy store
    // for the other samples to run?
    private static boolean bStandalone      = false;

    // Set if 4.x Agent needs to be created or 5.x Agent
    private static boolean bIs5xAgent       = false;

    static
    {
        // Initialize resource bundle
        try
        {
            bundle = ResourceBundle.getBundle(BUNDLE_NAME);
            SAMPLE_DESC     = bundle.getString("SAMPLE_DESC");
            OBJNAME_PREFIX  = bundle.getString("OBJNAME_PREFIX");
            STATUS_OK       = bundle.getString("STATUS_OK");
            STATUS_NOK      = bundle.getString("STATUS_NOK");
        }
        catch (MissingResourceException mre)
        {
            LogFatal("Failed to load the resource bundle " + BUNDLE_NAME);
        }
        catch (ClassCastException cce)
        {
            LogFatal("Failed to load the resource bundle " + BUNDLE_NAME);
        }
        catch (NullPointerException npe)
        {
            LogFatal("Failed to load the resource bundle.");
        }
    }


    public static void
    main(String[] args)
    {
        PolicyApiSample policyApiSample = new PolicyApiSample();

        if (args.length > 2)
        {
            LogFatal("Wrong number of arguments.");
        }

        // Parse command line arguments
        for (int i = 0; i < args.length; ++i)
        {
            // '-l' enables logging to a file
            if ((args[i].trim()).equalsIgnoreCase("-l"))
            {
                try
                {
                    String logFile = bundle.getString("LOGFILE_NAME");
                    logWriter = new PrintWriter(new BufferedWriter(new FileWriter(logFile)));
                }
                catch (IOException ioe)
                {
                    LogException(ioe);
                    logWriter = null;   // no logging
                }

                break;
            }
            else if ((args[i].trim()).equalsIgnoreCase("agent5x"))
            {
                bIs5xAgent = true;

                break;
            }
        }

        Log(CRLF + CRLF + bundle.getString("POLICYAPI_HEADER") + CRLF);
        Log(CRLF + CRLF + bundle.getString("ATTEMPT_CONNECT"));

        // Initialize connection settings
        AgentAPI agentapi               = new AgentAPI();
        ServerDef serverdef             = new ServerDef();
        serverdef.serverIpAddress       = bundle.getString("PS_IP");
        serverdef.connectionMin         = new Integer(bundle.getString("PS_CONMIN")).intValue();
        serverdef.connectionMax         = new Integer(bundle.getString("PS_CONMAX")).intValue();
        serverdef.connectionStep        = new Integer(bundle.getString("PS_CONSTEP")).intValue();
        serverdef.timeout               = new Integer(bundle.getString("PS_TIMEOUT")).intValue();
        serverdef.authenticationPort    = new Integer(bundle.getString("PS_AUPORT")).intValue();
        serverdef.authorizationPort     = new Integer(bundle.getString("PS_AZPORT")).intValue();
        serverdef.accountingPort        = new Integer(bundle.getString("PS_ACPORT")).intValue();

        InitDef initdef = new InitDef(bundle.getString("AGENT_NAME"),
                                      bundle.getString("AGENT_SECRET"),
                                      false,
                                      serverdef);

        int status = agentapi.init(initdef);

        if (status != 0)
        {
            Log(STATUS_NOK);
            LogFatal(bundle.getString("FAILED_CONNECT"));
        }

        Log(STATUS_OK);
        Log(CRLF + bundle.getString("ATTEMPT_AUTHADMIN"));
        SmApiSession session = new SmApiSession(new SmApiConnection(agentapi));

        // Session login
        try
        {
            SmApiResult result = session.login(bundle.getString("ADMIN_NAME"),
                                               bundle.getString("ADMIN_PWD"),
                                               InetAddress.getLocalHost(),
                                               0);

            if (!result.isSuccess())
            {
                Log(STATUS_NOK);
                LogFatal(bundle.getString("FAILED_AUTHADMIN") + " \'" + bundle.getString("ADMIN_NAME") + "\'");
            }

            Log(STATUS_OK);
        }
        catch (UnknownHostException uhe)
        {
            Log(STATUS_NOK);
            LogFatal(bundle.getString("FAILED_LOCALHOST"));
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
            LogFatal(bundle.getString("FAILED_AUTHADMIN") + " \'" + bundle.getString("ADMIN_NAME") + "\'");
        }

        SmPolicyApi policyapi = new SmPolicyApiImpl(session);

        // Create the policy store
        policyApiSample.resetPolicyStore(policyapi);
        policyApiSample.createSystemObjects(policyapi);
        policyApiSample.createDomainObjects(policyapi);
        policyApiSample.getSetGlobalSettings(policyapi);
        policyApiSample.getChildren(policyapi);
        policyApiSample.getVariableTypes(policyapi);
        policyApiSample.ManageKeys(policyapi);

        Log(CRLF + CRLF + bundle.getString("SAMPLE_FOOTER") + CRLF + CRLF);

        if (logWriter != null)
        {
            logWriter.close();
        }
    }


    private void
    createSystemObjects(SmPolicyApi policyapi)
    {
        SmApiResult result = null;

        // Create admin
        try
        {
            Log(CRLF + bundle.getString("CREATE_ADMIN"));
            result = this.createAdmin(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create agent
        try
        {
            Log(CRLF + bundle.getString("CREATE_AGENT"));
            result = this.createAgent(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create agent group
        try
        {
            Log(CRLF + bundle.getString("CREATE_AGENTGROUP"));
            result = this.createAgentGroup(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create Agent Config Object
        try
        {
            Log(CRLF + bundle.getString("CREATE_AGENTCONFIG"));
            result = this.createAgentConfig(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create Host Config Object
        try
        {
            Log(CRLF + bundle.getString("CREATE_HOSTCONFIG"));
            result = this.createHostConfig(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        //Create Trusted Host
        try
        {
            Log(CRLF + bundle.getString("CREATE_TRUSTEDHOST"));
            result = this.createTrustedHost(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (IOException ioe)
        {
            Log(STATUS_NOK);
            LogException(ioe);
        }
        catch (InterruptedException intre)
        {
            Log(STATUS_NOK);
            LogException(intre);
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create authentication scheme
        try
        {
            Log(CRLF + bundle.getString("CREATE_AUTHSCHEME"));
            result = this.createAuthScheme(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create LDAP user directory
        try
        {
            Log(CRLF + bundle.getString("CREATE_LDAPUSERDIR"));
            result = this.createLDAPUserDir(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            } else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create domain
        try
        {
            Log(CRLF + bundle.getString("CREATE_DOMAIN"));
            result = this.createDomain(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create registration scheme
        try
        {
            Log(CRLF + bundle.getString("CREATE_REGSCHEME"));
            result = this.createRegScheme(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create password policy
        try
        {
            Log(CRLF + bundle.getString("CREATE_PASSWDPOLICY"));
            result = this.createPwdPolicy(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create SQL Query Scheme
        try
        {
            Log(CRLF + bundle.getString("CREATE_SQLSCHEME"));
            result = this.createSQLQueryScheme(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Add user directory to domain
        try
        {
            Log(CRLF + bundle.getString("ADD_USERDIR_TO_DOMAIN"));
            result = policyapi.addUserDirToDomain(OBJNAME_PREFIX + "userdir",
                                                  OBJNAME_PREFIX + "domain");

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Add admin to domain
        try
        {
            Log(CRLF + bundle.getString("ADD_ADMIN_TO_DOMAIN"));
            result = policyapi.addAdminToDomain(OBJNAME_PREFIX + "admin",
                                                OBJNAME_PREFIX + "domain");

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Add agent to agent group
        try
        {
            Log(CRLF + bundle.getString("ADD_AGENT_TO_GROUP"));
            result = policyapi.addToGroup(SmGroup.Agent_Group_Prop,
                                          OBJNAME_PREFIX + "agent",
                                          OBJNAME_PREFIX + "agentgroup",
                                          "");

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create cerfiticate mapping
        try
        {
            Log(CRLF + bundle.getString("CREATE_CERTMAP"));
            result = this.createCertMap(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create auth-az directory mapping
        try
        {
            Log(CRLF + bundle.getString("CREATE_AUTHAZMAP"));
            result = this.createDirMap(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }
    }


    private void
    createDomainObjects(SmPolicyApi policyapi)
    {
        SmApiResult result = null;

        // Create realm
        try
        {
            Log(CRLF + bundle.getString("CREATE_REALM"));
            result = this.createRealm(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create rule
        try
        {
            Log(CRLF + bundle.getString("CREATE_RULE"));
            result = this.createRule(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }
        // Create global rule
        try
        {
            Log(CRLF + bundle.getString("CREATE_RULE"));
            result = this.createGlobalRule(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create rule group
        try
        {
            Log(CRLF + bundle.getString("CREATE_RULEGROUP"));
            result = this.createRuleGroup(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Add rule to rule group
        try
        {
            Log(CRLF + bundle.getString("ADD_RULE_TO_GROUP"));
            result = this.addRuleToGroup(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create response
        try
        {
            Log(CRLF + bundle.getString("CREATE_RESPONSE"));
            result = this.createResponse(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
                Log(CRLF + bundle.getString("CREATE_RESPONSEATTRS"));
                this.createResponseAttributes(policyapi);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }
        // Create response
        try
        {
            Log(CRLF + bundle.getString("CREATE_RESPONSE"));
            result = this.createGlobalResponse(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
                Log(CRLF + bundle.getString("CREATE_RESPONSEATTRS"));
                this.createGlobalResponseAttributes(policyapi);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }


        // Create response group
        try
        {
            Log(CRLF + bundle.getString("CREATE_RESPONSEGROUP"));
            result = this.createResponseGroup(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Add response to response group
        try
        {
            Log(CRLF + bundle.getString("ADD_RESPONSE_TO_GROUP"));
            result = this.addResponseToGroup(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create policy
        try
        {
            Log(CRLF + bundle.getString("CREATE_POLICY"));
            result = this.createPolicy(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create Global policy
        try
        {
            Log(CRLF + bundle.getString("CREATE_POLICY"));
            result = this.createGlobalPolicy(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Create Variables
        try
        {
            Log(CRLF + bundle.getString("CREATE_VARIABLE"));
            result = this.createVariable(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }
    }
    private void
    getSetGlobalSettings(SmPolicyApi policyapi)
    {
        SmRootConfig objRootConfig = new SmRootConfig();
        Vector RootConfigOID = new Vector();
        SmApiResult result = new SmApiResult();
        try
        {
            Log(CRLF + bundle.getString("MODIFY_NESTEDSECURITY"));
            policyapi.getGlobalObjectNames("RootConfig", RootConfigOID);
            SmObjId objID = SmObjId.valueOf(RootConfigOID.elementAt(0).toString());
            policyapi.getRootConfig(objID, objRootConfig);
            objRootConfig.setNestedSecurity(true);
            policyapi.modifyRootConfig(objRootConfig);
            result=policyapi.getRootConfig(objID, objRootConfig);
            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
            policyapi.getRootConfig(objID, objRootConfig);
            objRootConfig.setADEnhanced(true);
            policyapi.modifyRootConfig(objRootConfig);
            policyapi.getRootConfig(objID, objRootConfig);
            Log(CRLF +"Set ADEnhanced to " + objRootConfig.getADEnhanced());
        }
        catch(Exception e)
        {
        }
    }

    private void
    ManageKeys (SmPolicyApi policyapi)
    {
        SmKeyManagement keyManagement = new SmKeyManagement ();
        SmSharedSecretPolicy sharedSecretPolicy = new SmSharedSecretPolicy ();
        SmApiResult result = null;

        //Shared Secret Policy -- Get, modify Shared Secret Policy and change Shared Secret Key
        Log (CRLF + bundle.getString ("GET_SHARED_SECRET"));
        try
        {
            Vector SharedSecretPolicyOID = new Vector ();
            policyapi.getGlobalObjectNames (SmSharedSecretPolicy.PropSharedSecretPolicy, SharedSecretPolicyOID);
            result = policyapi.getSharedSecretPolicy (SharedSecretPolicyOID.elementAt(0).toString(), sharedSecretPolicy);
            if (result.isSuccess ())
            {
                Log (STATUS_OK);
            }
            else
            {
                Log (STATUS_NOK);
                LogError (result.getMessage ());
            }
        }
        catch (Exception e)
        {
            Log (e.getMessage ());
        }

        // Modify Shared Secret Policy
        Log (CRLF + bundle.getString ("MODIFY_SHARED_SECRET"));
        try
        {
            sharedSecretPolicy.setIsEnabled (true);
            sharedSecretPolicy.setRolloverPeriod (4);
            sharedSecretPolicy.setRolloverFrequency (5);
            result = policyapi.modifySharedSecretPolicy (sharedSecretPolicy);
            if (result.isSuccess ())
            {
                Log (STATUS_OK);
            }
            else
            {
                Log (STATUS_NOK);
                LogError (result.getMessage());
            }
        }
        catch (Exception e)
        {
            Log (e.getMessage ());
        }

        // ChangeSharedSecretKey
        Log (CRLF + bundle.getString ("ROLL_OVER_SHARED_SECRET"));
        try
        {
            result = policyapi.changeSharedSecretKey ();
            if (result.isSuccess ())
            {
                Log (STATUS_OK);
            }
            else
            {
                Log (STATUS_NOK);
                LogError (result.getMessage ());
            }
        }
        catch (Exception e)
        {
            Log (e.getMessage ());
        }

        // Key Management -- Get and modify Key management object
        Log (CRLF + bundle.getString ("GET_KEYMANAGEMENT"));
        try
        {

            Vector KeyManagementOID = new Vector ();
            policyapi.getGlobalObjectNames (SmKeyManagement.PropKeyManagement, KeyManagementOID);
            result = policyapi.getKeyManagement (KeyManagementOID.elementAt(0).toString(), keyManagement);
            if (result.isSuccess ())
            {
                Log (STATUS_OK);
            }
            else
            {
                Log (STATUS_NOK);
                LogError (result.getMessage ());
            }
        }
        catch (Exception e)
        {
            Log (e.getMessage ());
        }

        // Modify Key Management
        Log (CRLF + bundle.getString ("MODIFY_KEYMANAGEMENT"));
        try
        {
            keyManagement.setIsEnabled (true);
            keyManagement.setChangeFrequency (2);
            keyManagement.setFireHour (3);
            result = policyapi.modifyKeyManagement (keyManagement);
            if (result.isSuccess ())
            {
                Log (STATUS_OK);
            }
            else
            {
                Log (STATUS_NOK);
                LogError (result.getMessage());
            }
        }
        catch (Exception e)
        {
            Log (e.getMessage ());
        }
    }

    private void
    getChildren(SmPolicyApi policyapi)
    {
        try
        {
            SmDomain dmn = new SmDomain();
            SmApiResult result = policyapi.getDomain(OBJNAME_PREFIX + "domain", dmn);
            if (!result.isSuccess())
            {
                Log(CRLF + "Failed to find Domain");
                return;
            }
            Vector child = new Vector();
            result = policyapi.getChildren(dmn.getOid().toString(), child);
            Log(CRLF + child);
         }
         catch(Exception e)
         {
         }
    }
    // get Variable Types
    private void
    getVariableTypes (SmPolicyApi policyapi)
    {
        try 
        {
            Log(CRLF + bundle.getString("GET_VARIABLETYPES"));
            SmVariableType varType = new SmVariableType();
            SmApiResult result = policyapi.getVariableType(SmVariableType.FEDERATION_ATTRIBUTE_VARIABLES, varType);
            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (Exception e) {
        }
    }

    private void
    getGlobalObjects(SmPolicyApi policyapi)
    {
        SmApiResult result = null;

        // Get rule
        try
        {
            Log(CRLF + bundle.getString("GET_RULE"));
            result = this.getGlobalRule(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }

        // Get response
        try
        {
            Log(CRLF + bundle.getString("GET_RESPONSE"));
            result = this.getGlobalResponse(policyapi);

            if (result.isSuccess())
            {
                Log(STATUS_OK);
                Log(CRLF + bundle.getString("CREATE_RESPONSEATTRS"));
                this.createResponseAttributes(policyapi);
            }
            else
            {
                Log(STATUS_NOK);
                LogError(result.getMessage());
            }
        }
        catch (SmApiException sme)
        {
            Log(STATUS_NOK);
            LogException(sme);
        }
    }

    private SmApiResult
    createAdmin(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmAdmin admin = new SmAdmin();
        admin.setName(OBJNAME_PREFIX + "admin");
        admin.setDescription(SAMPLE_DESC + "admin");
        admin.setPassword(bundle.getString("ADMIN_PWD"));
        admin.setAdminRights(SmAdmin.AdminRightsManageAllDomains |
                             SmAdmin.AdminRightsManageObjects);
        admin.setDirAuth(false);

        return policyapi.addAdmin(admin);
    }

    private SmApiResult
    createAgent(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmAgent agent = new SmAgent();
        agent.setName(OBJNAME_PREFIX + "agent");
        agent.setDescription(SAMPLE_DESC + "agent");
        agent.setAgentType(SmAgentType.DefaultAgentType);

        if (bIs5xAgent)         // If 5.x Agent needs to be created
        {
            agent.setIpAddr("");
            agent.setSecret("");
        }
        else                    // If 4.x Agent needs to be created
        {
            agent.setIpAddr(bundle.getString("AGENT_IP"));
            agent.setSecret(bundle.getString("AGENT_SECRET"));
        }

        return policyapi.addAgent(agent);
    }


    private SmApiResult
    createAgentGroup(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmAgentGroup agentGroup = new SmAgentGroup();
        agentGroup.setName(OBJNAME_PREFIX + "agentgroup");
        agentGroup.setDescription(SAMPLE_DESC + "agentgroup");
        agentGroup.setAgentType(SmAgentType.DefaultAgentType);

        return policyapi.addGroup("",         // agent group domain name is empty.
                                  agentGroup);
    }


    private SmApiResult
    createAgentConfig(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmAgentConfig agentconfig = new SmAgentConfig();
        agentconfig.setName(OBJNAME_PREFIX + "agentconfig");
        agentconfig.setDescription(SAMPLE_DESC + "agentconfig");

        SmProperty properties[] = new SmProperty[1];

        String agentName = OBJNAME_PREFIX + "agent";
        properties[0] = new SmProperty("DefaultAgentName", agentName, new Integer(bundle.getString("ENCRYPTED")).intValue());

        agentconfig.setAgentConfigProperties(properties);

        return policyapi.addAgentConfig(agentconfig);
    }


    private SmApiResult
    createHostConfig(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmHostConfig hostconfobj = new SmHostConfig();
        hostconfobj.setName(OBJNAME_PREFIX + "hostconfig");
        hostconfobj.setServers(bundle.getString("POLICYSERVERLIST"));
        hostconfobj.setClusters(bundle.getString("CLUSTERS"));
        hostconfobj.setFailoverThreshold(new Integer(bundle.getString("FAILOVERTHRESHOLD")).intValue());
        hostconfobj.setEnableFailOver(new Integer(bundle.getString("ENABLEFAILOVER")).intValue());
        hostconfobj.setMaxSocketPerPort(new Integer(bundle.getString("MAXSOCKETSPERPORT")).intValue());
        hostconfobj.setMinSocketPerPort(new Integer(bundle.getString("MINSOCKETSPERPORT")).intValue());
        hostconfobj.setNewSocketStep(new Integer(bundle.getString("NEWSOCKETSTEP")).intValue());
        hostconfobj.setRequestTimeout(new Integer(bundle.getString("REQUESTTIMEOUT")).intValue());
        hostconfobj.setDescription(SAMPLE_DESC + "hostconfobj");

        return policyapi.addHostConfig(hostconfobj);
    }


    private SmApiResult
    createTrustedHost(SmPolicyApi policyapi)
        throws SmApiException, IOException, InterruptedException
    {
        SmTrustedHost trustedhost = new SmTrustedHost();
        trustedhost.setName(OBJNAME_PREFIX + "trustedhost");
        trustedhost.setIPAddress(bundle.getString("PS_IP"));
        trustedhost.setAdmin(bundle.getString("ADMIN_NAME"));
        trustedhost.setPassword(bundle.getString("ADMIN_PWD"));
        trustedhost.setHostConfigObjName(OBJNAME_PREFIX + "hostconfig");
        trustedhost.setHostConfigFile(bundle.getString("HOSTCONFIGFILE"));
        trustedhost.setEnableRollOver(new Integer(bundle.getString("ENABLEROLLOVER")).intValue());

        return policyapi.addTrustedHost(trustedhost);
    }

    private SmApiResult
    createAuthScheme(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmScheme basicScheme = new SmScheme();
        basicScheme.setName(OBJNAME_PREFIX + "authscheme");
        basicScheme.setDescription(SAMPLE_DESC + "authscheme");
        basicScheme.setIsUsedByAdmin(false);
        basicScheme.setType(SmScheme.TypeBasic);
        basicScheme.setLevel(7);
        basicScheme.setLibrary("smauthdir");

        return policyapi.addScheme(basicScheme);
    }


    private SmApiResult
    createLDAPUserDir(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmUserDirectory userdir = new SmUserDirectory();

        if (((bundle.getString("LDAP_USERDIR_USECREDS")).trim()).equals("1"))
        {
            userdir.setRequireCredentials(true);
            userdir.setUsername(bundle.getString("LDAP_USERDIR_CREDS_USERNAME"));
            userdir.setPassword(bundle.getString("LDAP_USERDIR_CREDS_PWD"));
        }

        userdir.setName(OBJNAME_PREFIX + "userdir");
        userdir.setDescription(SAMPLE_DESC + "userdir");
        userdir.setNamespace(SmUserDirectory.LDAP_NAMESPACE);
        userdir.setServer(bundle.getString("LDAP_USERDIR_IP"));
        userdir.setSearchRoot(bundle.getString("LDAP_USERDIR_SRCH_ROOT"));
        userdir.setUserLookupStart(bundle.getString("LDAP_USERDIR_LKUPSTRT"));
        userdir.setUserLookupEnd(bundle.getString("LDAP_USERDIR_LKUPEND"));
        userdir.setSearchResults(new Integer(bundle.getString("LDAP_SEARCH_RESULTS")).intValue());
        userdir.setSearchScope(new Integer(bundle.getString("LDAP_SEARCH_SCOPE")).intValue());
        userdir.setSearchTimeout(new Integer(bundle.getString("LDAP_SEARCH_TIMEOUT")).intValue());
        userdir.setUniversalIDAttr(bundle.getString("LDAP_UID_ATTR"));
        userdir.setDisabledAttr(bundle.getString("LDAP_DISABLED_ATTR"));
        userdir.setPasswordAttr(bundle.getString("LDAP_PASSWORD_ATTR"));
        userdir.setPasswordDataAttr(bundle.getString("LDAP_PASSWORD_DATA_ATTR"));

        return policyapi.addUserDirectory(userdir);
    }


    private SmApiResult
    createDomain(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain ();
        domain.setName(OBJNAME_PREFIX + "domain");
        domain.setDescription(SAMPLE_DESC + "domain");

        return policyapi.addDomain(domain);
    }


    private SmApiResult
    createSQLQueryScheme(SmPolicyApi policyapi)
        throws SmApiException
    {
        final String ODBC_PROP_AUTHENTICATEUSER     = bundle.getString("ODBC_PROP_AUTHENTICATEUSER");
        final String ODBC_PROP_ENUMERATE            = bundle.getString("ODBC_PROP_ENUMERATE");
        final String ODBC_PROP_GETGROUPPROP         = bundle.getString("ODBC_PROP_GETGROUPPROP");
        final String ODBC_PROP_GETGROUPPROPS        = bundle.getString("ODBC_PROP_GETGROUPPROPS");
        final String ODBC_PROP_GETGROUPS            = bundle.getString("ODBC_PROP_GETGROUPS");
        final String ODBC_PROP_GETOBJINFO           = bundle.getString("ODBC_PROP_GETOBJINFO");
        final String ODBC_PROP_GETUSERPROP          = bundle.getString("ODBC_PROP_GETUSERPROP");
        final String ODBC_PROP_GETUSERPROPS         = bundle.getString("ODBC_PROP_GETUSERPROPS");
        final String ODBC_PROP_INITUSER             = bundle.getString("ODBC_PROP_INITUSER");
        final String ODBC_PROP_QUERY_ISGROUPMEMBER  = bundle.getString("ODBC_PROP_QUERY_ISGROUPMEMBER");
        final String ODBC_PROP_LOOKUP               = bundle.getString("ODBC_PROP_LOOKUP");
        final String ODBC_PROP_LOOKUPGROUP          = bundle.getString("ODBC_PROP_LOOKUPGROUP");
        final String ODBC_PROP_LOOKUPUSER           = bundle.getString("ODBC_PROP_LOOKUPUSER");
        final String ODBC_PROP_SETGROUPPROP         = bundle.getString("ODBC_PROP_SETGROUPPROP");
        final String ODBC_PROP_SETPASSWORD          = bundle.getString("ODBC_PROP_SETPASSWORD");
        final String ODBC_PROP_SETUSERPROP          = bundle.getString("ODBC_PROP_SETUSERPROP");

        SmODBCQuery odbcquery = new SmODBCQuery();
        odbcquery.setName(OBJNAME_PREFIX + "queryscheme");
        odbcquery.setDescription(SAMPLE_DESC + "queryscheme");
        odbcquery.setQueryAuthenticateUser(ODBC_PROP_AUTHENTICATEUSER);
        odbcquery.setQueryEnumerate(ODBC_PROP_ENUMERATE);
        odbcquery.setQueryGetGroupProp(ODBC_PROP_GETGROUPPROP);
        odbcquery.setQueryGetGroupProps(ODBC_PROP_GETGROUPPROPS);
        odbcquery.setQueryGetGroups(ODBC_PROP_GETGROUPS);
        odbcquery.setQueryGetObjInfo(ODBC_PROP_GETOBJINFO);
        odbcquery.setQueryGetUserProp(ODBC_PROP_GETUSERPROP);
        odbcquery.setQueryGetUserProps(ODBC_PROP_GETUSERPROPS);
        odbcquery.setQueryInitUser(ODBC_PROP_INITUSER);
        odbcquery.setQueryIsGroupMember(ODBC_PROP_QUERY_ISGROUPMEMBER);
        odbcquery.setQueryLookup(ODBC_PROP_LOOKUP);
        odbcquery.setQueryLookupGroup(ODBC_PROP_LOOKUPGROUP);
        odbcquery.setQueryLookupUser(ODBC_PROP_LOOKUPUSER);
        odbcquery.setQuerySetGroupProp(ODBC_PROP_SETGROUPPROP);
        odbcquery.setQuerySetPassword(ODBC_PROP_SETPASSWORD);
        odbcquery.setQuerySetUserProp(ODBC_PROP_SETUSERPROP);

        return policyapi.addODBCQuery(odbcquery);
    }


    private SmApiResult
    createRegScheme(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmUserDirectory userdir = new SmUserDirectory();
        policyapi.getUserDirectory(OBJNAME_PREFIX + "userdir", userdir);

        SmSelfReg regscheme = new SmSelfReg();
        regscheme.setName(OBJNAME_PREFIX + "regscheme");
        regscheme.setDescription(SAMPLE_DESC + "regscheme");
        regscheme.setEnableLogging(false);
        regscheme.setUserDir(userdir.getOid());
        regscheme.setRegistrationURL(bundle.getString("REG_WORKFLOW_LIB"));
        regscheme.setTemplatePath(bundle.getString("REG_TEMPLATE_PATH"));
        regscheme.setWelcomePathProvider(bundle.getString("REG_REGISTRATION_URL"));

        return policyapi.addSelfReg(regscheme);
    }


    private SmApiResult
    createPwdPolicy(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmUserDirectory userdir = new SmUserDirectory();
        policyapi.getUserDirectory(OBJNAME_PREFIX + "userdir", userdir);

        SmPasswordPolicy pwdPolicy = new SmPasswordPolicy();
        pwdPolicy.setUserDirectoryOid(userdir.getOid());
        pwdPolicy.setName(OBJNAME_PREFIX + "pwdpolicy");
        pwdPolicy.setDescription(SAMPLE_DESC + "pwdpolicy");
        pwdPolicy.setEnabled(true);
        pwdPolicy.setEntireDir(true);
        pwdPolicy.setDictionaryName(bundle.getString("PWD_DICTIONARY_NAME"));
        pwdPolicy.setDictionaryPartial(new Integer(bundle.getString("PWD_DICTIONARY_PARTIAL")).intValue());
        pwdPolicy.setExpirationDelay(new Integer(bundle.getString("PWD_EXPIRATION_DELAY")).intValue());
        pwdPolicy.setFilterClass(bundle.getString("PWD_FILTER_CLASS"));
        pwdPolicy.setFilterPath(bundle.getString("PWD_FILTER_PATH"));
        pwdPolicy.setLoginDaysGrace(new Integer(bundle.getString("PWD_LOGIN_DAYS_GRACE")).intValue());
        pwdPolicy.setLoginInactivityWarn(new Integer(bundle.getString("PWD_LOGIN_INACTIVITY_WARN")).intValue());
        pwdPolicy.setLoginMaxFailures(new Integer(bundle.getString("PWD_LOGIN_MAX_FAILURES")).intValue());
        pwdPolicy.setLoginMaxInactivity(new Integer(bundle.getString("PWD_LOGIN_MAX_INACTIVITY")).intValue());
        pwdPolicy.setPasswordMaxLength(new Integer(bundle.getString("PWD_PASSWORD_MAX_LENGTH")).intValue());
        pwdPolicy.setPasswordMaxRepeatingChar(new Integer(bundle.getString("PWD_PASSWORD_MAX_REPEAT")).intValue());
        pwdPolicy.setPasswordMinAlpha(new Integer(bundle.getString("PWD_PASSWORD_MIN_ALPHA")).intValue());
        pwdPolicy.setPasswordMinAlphaNum(new Integer(bundle.getString("PWD_PASSWORD_MIN_ALPHANUM")).intValue());
        pwdPolicy.setPasswordMinLength(new Integer(bundle.getString("PWD_PASSWORD_MIN_LENGTH")).intValue());
        pwdPolicy.setPasswordMinNonAlpha(new Integer(bundle.getString("PWD_PASSWORD_MIN_NONALPHA")).intValue());
        pwdPolicy.setPasswordMinNonPrintable(new Integer(bundle.getString("PWD_PASSWORD_MIN_NONPRINTABLE")).intValue());
        pwdPolicy.setPasswordMinNumbers(new Integer(bundle.getString("PWD_PASSWORD_MIN_NUMBERS")).intValue());
        pwdPolicy.setPasswordMinPunctuation(new Integer(bundle.getString("PWD_PASSWORD_MIN_PUNCTUATION")).intValue());
        pwdPolicy.setPasswordPercentDifferent(new Integer(bundle.getString("PWD_PASSWORD_PERCENT_DIFF")).intValue());
        pwdPolicy.setPasswordPercentSequence(new Integer(bundle.getString("PWD_PASSWORD_PERCENT_SEQUENCE")).intValue());
        pwdPolicy.setPasswordReuseCount(new Integer(bundle.getString("PWD_PASSWORD_REUSE_COUNT")).intValue());
        pwdPolicy.setPasswordReuseDelay(new Integer(bundle.getString("PWD_PASSWORD_REUSE_DELAY")).intValue());
        pwdPolicy.setPasswordServicesRedirect(bundle.getString("PWD_PASSWORD_SRVCS_REDIRECT"));
        pwdPolicy.setPasswordSpecialsLength(new Integer(bundle.getString("PWD_PASSWORD_SPECIALS_LENGTH")).intValue());
        pwdPolicy.setReenablement(new Integer(bundle.getString("PWD_REENABLEMENT")).intValue());
        pwdPolicy.setResolution(new Integer(bundle.getString("PWD_RESOLUTION")).intValue());

        int passwordBehavior = SmPasswordPolicy.Sm_PasswordPolicy_DontTrackLogins |
                               SmPasswordPolicy.Sm_PasswordPolicy_AllowFailedWrites |
                               SmPasswordPolicy.Sm_PasswordPolicy_InactivityForcePWChange |
                               SmPasswordPolicy.Sm_PasswordPolicy_PWExpiredForcePWChange;
        pwdPolicy.setPasswordBehavior(passwordBehavior);

        return policyapi.addPasswordPolicy(pwdPolicy);
    }


    private SmApiResult
    createCertMap(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmUserDirectory userdir = new SmUserDirectory();
        policyapi.getUserDirectory(OBJNAME_PREFIX + "userdir", userdir);

        SmCertMap certmap = new SmCertMap();
        certmap.setUserDir(userdir.getOid());
        certmap.setIssuerDN(bundle.getString("CERT_ISSUER_DN"));
        certmap.setCADN(bundle.getString("CERT_CADN"));

        int flags = SmCertMap.CertMapFlags_CertRequired             |
                    SmCertMap.CertMapFlags_UseDistributionPoints    |
                    SmCertMap.CertMapFlags_VerifySignature          |
                    SmCertMap.CertMapFlags_CRLCheck                 |
                    SmCertMap.CertMapFlags_Cache;

        certmap.setFlags(flags);
        certmap.setAttrType(SmCertMap.CertMapAttrType_Single);
        certmap.setMapAttr(bundle.getString("CERT_MAP_ATTR"));
        certmap.setDirType(SmCertMap.DirType_LDAP);

        return policyapi.addCertMap(certmap);
    }


    private SmApiResult
    createDirMap(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmUserDirectory userdir = new SmUserDirectory();
        policyapi.getUserDirectory(OBJNAME_PREFIX + "userdir", userdir);

        SmAuthAzMap authAzMap = new SmAuthAzMap();
        authAzMap.setAuthUserDir(userdir.getOid());
        authAzMap.setAzUserDir(userdir.getOid());
        authAzMap.setAuthAzType(SmAuthAzMap.AUTHAZ_MAPTYPE_UNIVERSALID);

        return policyapi.addAuthAzMap(authAzMap);
    }


    private SmApiResult
    createRealm(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        SmAgent agent = new SmAgent();
        policyapi.getAgent(OBJNAME_PREFIX + "agent", agent);

        SmScheme authscheme = new SmScheme();
        policyapi.getScheme(OBJNAME_PREFIX + "authscheme", authscheme);

        SmSelfReg regscheme = new SmSelfReg();
        policyapi.getSelfReg(OBJNAME_PREFIX + "regscheme", regscheme);

        SmUserDirectory userdir = new SmUserDirectory();
        policyapi.getUserDirectory(OBJNAME_PREFIX + "userdir", userdir);

        SmRealm realm = new SmRealm();
        realm.setName(OBJNAME_PREFIX + "realm");
        realm.setDescription(SAMPLE_DESC + "realm");
        realm.setDomain(domain);
        realm.setResourceFilter(bundle.getString("REALM_RESOURCE_FILTER"));
        realm.setAgent(agent.getOid());
        realm.setAgentType(SmAgentType.DefaultAgentType);
        realm.setScheme(authscheme.getOid());
        realm.setProcessAuthEvents(true);
        realm.setProcessAzEvents(true);
        realm.setProtectAll(true);
        realm.setSelfRegOid(regscheme.getOid());
        realm.setAzUserDirOid(userdir.getOid());
        realm.setMaxTimeout(new Integer(bundle.getString("REALM_MAX_TIMEOUT")).intValue());
        realm.setIdleTimeout(new Integer(bundle.getString("REALM_IDLE_TIMEOUT")).intValue());
        realm.setParentRealmOid(domain.getOid());
        realm.setSyncAudit(false);

        return policyapi.addRealm(realm);
    }


    private SmApiResult
    createRule(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        SmRealm realm = new SmRealm();
        policyapi.getRealm(OBJNAME_PREFIX + "realm", domain.getName(), realm);

        SmRule rule = new SmRule();
        rule.setName(OBJNAME_PREFIX + "rule");
        rule.setDescription(SAMPLE_DESC + "rule");
        rule.setRealm(realm.getOid());
        rule.setDomain(domain);
        rule.setAgentType(SmAgentType.DefaultAgentType);
        rule.setAction(SmRule.ACTION_GET + "," + SmRule.ACTION_POST);
        rule.setResource("*");
        rule.setAllowAccess(true);
        rule.setRegularExpression(false);
        rule.setEnabled(true);
        rule.setActiveExpression(bundle.getString("RULE_ACTIVE_EXPRESSION"));

        SmTime time = new SmTime();
        byte timeGrid[] = new byte[24];

        for (int i = 0; i < 24; ++i)
        {
            timeGrid[i] = 0x55;
        }

        time.setTimeGrid(timeGrid);

        // Set end date
        Calendar toCal = Calendar.getInstance();
        toCal.set(Calendar.YEAR, new Integer(bundle.getString("RULE_END_YEAR")).intValue());
        toCal.set(Calendar.MONTH, new Integer(bundle.getString("RULE_END_MONTH")).intValue());
        toCal.set(Calendar.DAY_OF_MONTH, new Integer(bundle.getString("RULE_END_DAY")).intValue());
        time.setEndDate(toCal);
        rule.setTime(time);

        return policyapi.addRule(rule);
    }
    private SmApiResult
    createGlobalRule(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmRule rule = new SmRule();
        rule.setName(OBJNAME_PREFIX + "grule");
        rule.setDescription(SAMPLE_DESC + "grule");
        rule.setRealm(SmObjId.NULL_OID.toString());
        rule.setDomain(SmObjId.NULL_OID.toString());
        SmAgent agent = new SmAgent();
        policyapi.getAgent(OBJNAME_PREFIX + "agent",agent);
        rule.setAgent(agent.getOid().toString());
        rule.setAction(SmRule.ACTION_ON_AUTH_ACCEPT);
        rule.setResource("*");
        rule.setAllowAccess(true);
        rule.setRegularExpression(false);
        rule.setEnabled(true);
        rule.setActiveExpression(bundle.getString("RULE_ACTIVE_EXPRESSION"));
        SmTime time = new SmTime();
        byte timeGrid[] = new byte[24];

        for (int i = 0; i < 24; ++i)
        {
            timeGrid[i] = 0x55;
        }

        time.setTimeGrid(timeGrid);

        // Set end date
        Calendar toCal = Calendar.getInstance();
        toCal.set(Calendar.YEAR, new Integer(bundle.getString("RULE_END_YEAR")).intValue());
        toCal.set(Calendar.MONTH, new Integer(bundle.getString("RULE_END_MONTH")).intValue());
        toCal.set(Calendar.DAY_OF_MONTH, new Integer(bundle.getString("RULE_END_DAY")).intValue());
        time.setEndDate(toCal);
        rule.setTime(time);

        return policyapi.addRule(rule,true);
    }

    private SmApiResult
    getGlobalRule(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmRule rule = new SmRule();
        return policyapi.getRule(OBJNAME_PREFIX + "grule",SmObjId.NULL_OID.toString(),SmObjId.NULL_OID.toString(),rule);
    }

    private SmApiResult
    getGlobalResponse(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmResponse response = new SmResponse();
        return policyapi.getResponse(OBJNAME_PREFIX + "gresponse",SmObjId.NULL_OID.toString(),response);
    }

    private SmApiResult
    createRuleGroup(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmRuleGroup rulegroup = new SmRuleGroup();
        rulegroup.setName(OBJNAME_PREFIX + "rulegroup");
        rulegroup.setDescription(SAMPLE_DESC + "rulegroup");
        rulegroup.setAgentType(SmAgentType.DefaultAgentType);

        return policyapi.addGroup(OBJNAME_PREFIX + "domain", rulegroup);
    }


    private SmApiResult
    addRuleToGroup(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        return policyapi.addToGroup(SmGroup.Rule_Group_Prop,
                                    OBJNAME_PREFIX + "rule",
                                    OBJNAME_PREFIX + "rulegroup",
                                    domain.getOid().toString());
    }


    private SmApiResult
    createResponse(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmApiResult result;
        SmDomain domain = new SmDomain ();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);
        SmResponse response = new SmResponse();
        response.setName(OBJNAME_PREFIX + "response");
        response.setDescription(SAMPLE_DESC + "response");
        response.setAgentType(SmAgentType.DefaultAgentType);
        response.setDomain(domain);

        return policyapi.addResponse (response);
    }

    private SmApiResult
    createGlobalResponse(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmApiResult result;
        SmResponse response = new SmResponse();
        response.setName(OBJNAME_PREFIX + "gresponse");
        response.setDescription(SAMPLE_DESC + "gresponse");
        response.setAgentType(SmAgentType.DefaultAgentType);
        response.setDomain(SmObjId.NULL_OID.toString());
        return policyapi.addResponse (response,true);
    }

    private SmApiResult
    createResponseAttributes(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain ();
        SmResponse response = new SmResponse();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        SmApiResult result = policyapi.getResponse(OBJNAME_PREFIX + "response",
                                                   OBJNAME_PREFIX + "domain",
                                                   response);

        if (!result.isSuccess())
        {
            return result;
        }

        // Static WebAgent-HTTP-Header-Variable Response Attribute
        Log(CRLF + bundle.getString("CREATE_HEADER_VARIABLE"));
        SmResponseAttr attrWAhttpHdrVar = new SmResponseAttr();
        attrWAhttpHdrVar.setAgentTypeAttr(SmResponseAttr.ATTR_WA_HTTP_HEADER_VARIABLE);
        attrWAhttpHdrVar.setType(SmResponseAttr.AttrType_Active);
        attrWAhttpHdrVar.setResponse(response.getOid());
        attrWAhttpHdrVar.setDomain(domain);
        attrWAhttpHdrVar.setTTL(60);
        attrWAhttpHdrVar.setValue("varname=varvalue");

        if ((policyapi.addResponseAttr(attrWAhttpHdrVar)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-HTTP-Cookie-Variable Response Attribute
        Log(CRLF + bundle.getString("CREATE_COOKIE_VARIABLE"));
        SmResponseAttr attrWACookieVar = new SmResponseAttr();
        attrWACookieVar.setAgentTypeAttr(SmResponseAttr.ATTR_WA_HTTP_COOKIE_VARIABLE);
        attrWACookieVar.setType(SmResponseAttr.AttrType_Active);
        attrWACookieVar.setResponse(response.getOid());
        attrWACookieVar.setDomain(domain.getOid());
        attrWACookieVar.setValue("cookiename=cookievalue");

        if ((policyapi.addResponseAttr(attrWACookieVar)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAccept-Redirect Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_REDIRECT"));
        SmResponseAttr attrWAOnAcceptRedirect = new SmResponseAttr();
        attrWAOnAcceptRedirect.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONACCEPT_REDIRECT);
        attrWAOnAcceptRedirect.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAcceptRedirect.setResponse(response.getOid());
        attrWAOnAcceptRedirect.setDomain(domain);
        attrWAOnAcceptRedirect.setValue("redirvalue");

        if ((policyapi.addResponseAttr(attrWAOnAcceptRedirect)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAccept-Text Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_TEXT"));
        SmResponseAttr attrWAOnAcceptText = new SmResponseAttr();
        attrWAOnAcceptText.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONACCEPT_TEXT);
        attrWAOnAcceptText.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAcceptText.setResponse(response.getOid());
        attrWAOnAcceptText.setDomain(domain.getOid());
        attrWAOnAcceptText.setValue("onaccepttext");

        if ((policyapi.addResponseAttr(attrWAOnAcceptText)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAuthAccept-Session-Idle-Timeout Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_IDLE"));
        SmResponseAttr attrWAOnAuthAcceptSessionIdleTimeout = new SmResponseAttr();
        attrWAOnAuthAcceptSessionIdleTimeout.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONAUTHACCEPT_SESSION_IDLE_TIMEOUT);
        attrWAOnAuthAcceptSessionIdleTimeout.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAuthAcceptSessionIdleTimeout.setResponse(response.getOid());
        attrWAOnAuthAcceptSessionIdleTimeout.setDomain(domain);
        attrWAOnAuthAcceptSessionIdleTimeout.setValue("10");

        if ((policyapi.addResponseAttr(attrWAOnAuthAcceptSessionIdleTimeout)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAuthAccept-Session-Max-Timeout Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_MAX"));
        SmResponseAttr attrWAOnAuthAcceptSessionMaxTimeout = new SmResponseAttr();
        attrWAOnAuthAcceptSessionMaxTimeout.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONAUTHACCEPT_SESSION_MAX_TIMEOUT);
        attrWAOnAuthAcceptSessionMaxTimeout.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAuthAcceptSessionMaxTimeout.setResponse(response.getOid());
        attrWAOnAuthAcceptSessionMaxTimeout.setDomain(domain.getOid());
        attrWAOnAuthAcceptSessionMaxTimeout.setValue("60");

        if ((policyapi.addResponseAttr(attrWAOnAuthAcceptSessionMaxTimeout)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnReject-Redirect Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONREJECT_REDIRECT"));
        SmResponseAttr attrWAOnRejectRedirect = new SmResponseAttr();
        attrWAOnRejectRedirect.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONREJECT_REDIRECT);
        attrWAOnRejectRedirect.setType(SmResponseAttr.AttrType_Active);
        attrWAOnRejectRedirect.setResponse(response.getOid());
        attrWAOnRejectRedirect.setDomain(domain);
        attrWAOnRejectRedirect.setValue("http://some.redirval.com");

        if ((policyapi.addResponseAttr(attrWAOnRejectRedirect)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnReject-Redirect Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONREJECT_TEXT"));
        SmResponseAttr attrWAOnRejectText = new SmResponseAttr();
        attrWAOnRejectText.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONREJECT_TEXT);
        attrWAOnRejectText.setType(SmResponseAttr.AttrType_Active);
        attrWAOnRejectText.setResponse(response.getOid());
        attrWAOnRejectText.setDomain(domain.getOid());
        attrWAOnRejectText.setValue("rejecttext");

        if ((policyapi.addResponseAttr(attrWAOnRejectText)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        return new SmApiResult();
    }

    private SmApiResult
    createGlobalResponseAttributes(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmResponse response = new SmResponse();

        SmApiResult result = policyapi.getResponse(OBJNAME_PREFIX + "response",
                                                   SmObjId.NULL_OID.toString(),
                                                   response);

        if (!result.isSuccess())
        {
            return result;
        }

        // Static WebAgent-HTTP-Header-Variable Response Attribute
        Log(CRLF + bundle.getString("CREATE_HEADER_VARIABLE"));
        SmResponseAttr attrWAhttpHdrVar = new SmResponseAttr();
        attrWAhttpHdrVar.setAgentTypeAttr(SmResponseAttr.ATTR_WA_HTTP_HEADER_VARIABLE);
        attrWAhttpHdrVar.setType(SmResponseAttr.AttrType_Active);
        attrWAhttpHdrVar.setResponse(response);
        attrWAhttpHdrVar.setTTL(60);
        attrWAhttpHdrVar.setValue("varname=varvalue");

        if ((policyapi.addResponseAttr(attrWAhttpHdrVar,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-HTTP-Cookie-Variable Response Attribute
        Log(CRLF + bundle.getString("CREATE_COOKIE_VARIABLE"));
        SmResponseAttr attrWACookieVar = new SmResponseAttr();
        attrWACookieVar.setAgentTypeAttr(SmResponseAttr.ATTR_WA_HTTP_COOKIE_VARIABLE);
        attrWACookieVar.setType(SmResponseAttr.AttrType_Active);
        attrWACookieVar.setResponse(response);
        attrWACookieVar.setValue("cookiename=cookievalue");

        if ((policyapi.addResponseAttr(attrWACookieVar,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAccept-Redirect Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_REDIRECT"));
        SmResponseAttr attrWAOnAcceptRedirect = new SmResponseAttr();
        attrWAOnAcceptRedirect.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONACCEPT_REDIRECT);
        attrWAOnAcceptRedirect.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAcceptRedirect.setResponse(response);
        attrWAOnAcceptRedirect.setValue("redirvalue");

        if ((policyapi.addResponseAttr(attrWAOnAcceptRedirect,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAccept-Text Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_TEXT"));
        SmResponseAttr attrWAOnAcceptText = new SmResponseAttr();
        attrWAOnAcceptText.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONACCEPT_TEXT);
        attrWAOnAcceptText.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAcceptText.setResponse(response);
        attrWAOnAcceptText.setValue("onaccepttext");

        if ((policyapi.addResponseAttr(attrWAOnAcceptText,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAuthAccept-Session-Idle-Timeout Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_IDLE"));
        SmResponseAttr attrWAOnAuthAcceptSessionIdleTimeout = new SmResponseAttr();
        attrWAOnAuthAcceptSessionIdleTimeout.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONAUTHACCEPT_SESSION_IDLE_TIMEOUT);
        attrWAOnAuthAcceptSessionIdleTimeout.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAuthAcceptSessionIdleTimeout.setResponse(response);
        attrWAOnAuthAcceptSessionIdleTimeout.setValue("10");

        if ((policyapi.addResponseAttr(attrWAOnAuthAcceptSessionIdleTimeout,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnAuthAccept-Session-Max-Timeout Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONACCEPT_MAX"));
        SmResponseAttr attrWAOnAuthAcceptSessionMaxTimeout = new SmResponseAttr();
        attrWAOnAuthAcceptSessionMaxTimeout.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONAUTHACCEPT_SESSION_MAX_TIMEOUT);
        attrWAOnAuthAcceptSessionMaxTimeout.setType(SmResponseAttr.AttrType_Active);
        attrWAOnAuthAcceptSessionMaxTimeout.setResponse(response);
        attrWAOnAuthAcceptSessionMaxTimeout.setValue("60");

        if ((policyapi.addResponseAttr(attrWAOnAuthAcceptSessionMaxTimeout,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnReject-Redirect Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONREJECT_REDIRECT"));
        SmResponseAttr attrWAOnRejectRedirect = new SmResponseAttr();
        attrWAOnRejectRedirect.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONREJECT_REDIRECT);
        attrWAOnRejectRedirect.setType(SmResponseAttr.AttrType_Active);
        attrWAOnRejectRedirect.setResponse(response);
        attrWAOnRejectRedirect.setValue("http://some.redirval.com");

        if ((policyapi.addResponseAttr(attrWAOnRejectRedirect,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        // Static WebAgent-OnReject-Redirect Response Attribute
        Log(CRLF + bundle.getString("CREATE_ONREJECT_TEXT"));
        SmResponseAttr attrWAOnRejectText = new SmResponseAttr();
        attrWAOnRejectText.setAgentTypeAttr(SmResponseAttr.ATTR_WA_ONREJECT_TEXT);
        attrWAOnRejectText.setType(SmResponseAttr.AttrType_Active);
        attrWAOnRejectText.setResponse(response);
        attrWAOnRejectText.setValue("rejecttext");

        if ((policyapi.addResponseAttr(attrWAOnRejectText,true)).isSuccess())
        {
            Log(STATUS_OK);
        }
        else
        {
            Log(STATUS_NOK);
            LogError(result.getMessage());
        }

        return new SmApiResult();
    }

    private SmApiResult
    createResponseGroup(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmResponseGroup responseGroup = new SmResponseGroup();
        responseGroup.setName(OBJNAME_PREFIX + "responsegroup");
        responseGroup.setDescription(SAMPLE_DESC + "responsegroup");
        responseGroup.setAgentType(SmAgentType.DefaultAgentType);

        return policyapi.addGroup(OBJNAME_PREFIX + "domain", responseGroup);
    }


    private SmApiResult
    addResponseToGroup(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        return policyapi.addToGroup(SmGroup.Response_Group_Prop,
                                    OBJNAME_PREFIX + "response",
                                    OBJNAME_PREFIX + "responsegroup",
                                    domain.getOid().toString());
    }


    private SmApiResult
    createPolicy(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmDomain domain = new SmDomain();
        SmPolicy policy = new SmPolicy();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        policy.setName(OBJNAME_PREFIX + "policy");
        policy.setDescription(OBJNAME_PREFIX + "policy");
        policy.setEnabled(true);
        policy.setActiveExpression(bundle.getString("POLICY_ACTIVE_EXPRESSION"));
        policy.setDomain(domain);

        // Set IP Restrictions
        String agentIP = bundle.getString("AGENT_IP");

        if (agentIP.equals("127.0.0.1"))
        {
            try
            {
                agentIP = (InetAddress.getLocalHost()).getHostAddress();
            }
            catch (UnknownHostException uhe)
            {
                LogException(uhe);
            }
        }

        Vector ipRestrictions = new Vector();
        SmIPAddress ipRestriction1 = new SmIPAddress();
        ipRestriction1.setIPMode(SmIPAddress.IPADDRESS);
        ipRestriction1.setIPAddress(agentIP);
        ipRestrictions.addElement(ipRestriction1);

        SmIPAddress ipRestriction2 = new SmIPAddress();
        ipRestriction2.setIPMode(SmIPAddress.IPRANGE);
        ipRestriction2.setIPAddress("2.22.222.5");
        ipRestriction2.setEndIPAddress("2.22.222.8");
        ipRestrictions.addElement(ipRestriction2);

        SmIPAddress ipRestriction3 = new SmIPAddress();
        ipRestriction3.setIPAddress("2.22.222.10");
        ipRestriction3.setIPMode (SmIPAddress.SUBNET);
        ipRestriction3.setSubnetBits(26);
        ipRestrictions.addElement(ipRestriction3);

        SmIPAddress ipRestriction4 = new SmIPAddress();
        ipRestriction4.setIPMode(SmIPAddress.HOSTNAME);
        ipRestriction4.setHostname("hostname.yourcompany.com");
        ipRestrictions.addElement(ipRestriction4);

        policy.setIPRestriction(ipRestrictions);

        // Set time restrictions
        SmTime policyTimeRestriction = new SmTime();
        byte policyTimeGrid[] = new byte[24];

        for (int i = 0; i < 24; ++i)
        {
            policyTimeGrid[i] = 0x55;
        }

        policyTimeRestriction.setTimeGrid(policyTimeGrid);

        // Set end date
        Calendar policyEndDate = Calendar.getInstance();
        policyEndDate.set(Calendar.YEAR, new Integer(bundle.getString("POLICY_END_YEAR")).intValue());
        policyEndDate.set(Calendar.MONTH, new Integer(bundle.getString("POLICY_END_MONTH")).intValue());
        policyEndDate.set(Calendar.DAY_OF_MONTH, new Integer(bundle.getString("POLICY_END_DAY")).intValue());
        policyTimeRestriction.setEndDate(policyEndDate);
        policy.setTime(policyTimeRestriction);

        SmApiResult result = policyapi.addPolicy(policy);

        if (!result.isSuccess())
        {
            return result;
        }

        SmRealm realm = new SmRealm();
        result = policyapi.getRealm(OBJNAME_PREFIX + "realm", OBJNAME_PREFIX + "domain", realm);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_GET_REALM"));
        }

        SmRule rule = new SmRule();
        result = policyapi.getRule(OBJNAME_PREFIX + "rule",
                                   realm.getOid().toString(),
                                   OBJNAME_PREFIX + "domain",
                                   rule);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_GET_RULE"));
        }

        SmPolicyLink policyLink = new SmPolicyLink();
        policyLink.setPolicy(policy.getOid());
        policyLink.setDomain(domain);
        policyLink.setRule(rule.getOid());

        result = policyapi.addPolicyLink(policyLink);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_POLICY_LINK"));
        }

        // set Response to the policy link
        SmResponse response = new SmResponse();
        result = policyapi.getResponse(OBJNAME_PREFIX + "response",
                                       OBJNAME_PREFIX + "domain",
                                       response);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_GET_RESPONSE"));
        }

        policyLink.setResponse(response.getOid());
        result = policyapi.setResponseInPolicyLink(policyLink);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_SET_RESPONSE"));
        }

        SmUserDirectory userdir = new SmUserDirectory();
        policyapi.getUserDirectory(OBJNAME_PREFIX + "userdir", userdir);

        // Add user policy
        int nResolution = new Integer(bundle.getString("POLICY_RESOLUTION")).intValue();

        if (nResolution < 0 || nResolution > 10)
        {
            nResolution = 0;
        }

        SmUserPolicy userPolicy = new SmUserPolicy();
        userPolicy.setFilterPath(bundle.getString("POLICY_FILTER_PATH"));
        userPolicy.setFilterClass(bundle.getString("POLICY_FILTER_CLASS"));
        userPolicy.setPolicyResolution(nResolution);
        userPolicy.setPolicyFlags(0);
        userPolicy.setPolicy(policy.getOid());
        userPolicy.setDomain(domain.getOid());
        userPolicy.setUserDirectory(userdir.getOid());

        return policyapi.addUserPolicy(userPolicy);
    }

    private SmApiResult
    createGlobalPolicy(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmPolicy policy = new SmPolicy();
        policy.setName(OBJNAME_PREFIX + "gpolicy");
        policy.setDescription(OBJNAME_PREFIX + "gpolicy");
        policy.setEnabled(true);
        policy.setActiveExpression(bundle.getString("POLICY_ACTIVE_EXPRESSION"));

        // Set IP Restrictions
        String agentIP = bundle.getString("AGENT_IP");
        if (agentIP.equals("127.0.0.1"))
        {
            try
            {
                agentIP = (InetAddress.getLocalHost()).getHostAddress();
            }
            catch (UnknownHostException uhe)
            {
                LogException(uhe);
            }
        }

        Vector ipRestrictions = new Vector();
        SmIPAddress ipRestriction1 = new SmIPAddress();
        ipRestriction1.setIPMode(SmIPAddress.IPADDRESS);
        ipRestriction1.setIPAddress(agentIP);
        ipRestrictions.addElement(ipRestriction1);

        SmIPAddress ipRestriction2 = new SmIPAddress();
        ipRestriction2.setIPMode(SmIPAddress.IPRANGE);
        ipRestriction2.setIPAddress("2.22.222.5");
        ipRestriction2.setEndIPAddress("2.22.222.8");
        ipRestrictions.addElement(ipRestriction2);

        SmIPAddress ipRestriction3 = new SmIPAddress();
        ipRestriction3.setIPAddress("2.22.222.10");
        ipRestriction3.setIPMode (SmIPAddress.SUBNET);
        ipRestriction3.setSubnetBits(26);
        ipRestrictions.addElement(ipRestriction3);

        SmIPAddress ipRestriction4 = new SmIPAddress();
        ipRestriction4.setIPMode(SmIPAddress.HOSTNAME);
        ipRestriction4.setHostname("hostname.yourcompany.com");
        ipRestrictions.addElement(ipRestriction4);

        policy.setIPRestriction(ipRestrictions);

        // Set time restrictions
        SmTime policyTimeRestriction = new SmTime();
        byte policyTimeGrid[] = new byte[24];

        for (int i = 0; i < 24; ++i)
        {
            policyTimeGrid[i] = 0x55;
        }

        policyTimeRestriction.setTimeGrid(policyTimeGrid);

        // Set end date
        Calendar policyEndDate = Calendar.getInstance();
        policyEndDate.set(Calendar.YEAR, new Integer(bundle.getString("POLICY_END_YEAR")).intValue());
        policyEndDate.set(Calendar.MONTH, new Integer(bundle.getString("POLICY_END_MONTH")).intValue());
        policyEndDate.set(Calendar.DAY_OF_MONTH, new Integer(bundle.getString("POLICY_END_DAY")).intValue());
        policyTimeRestriction.setEndDate(policyEndDate);
        policy.setTime(policyTimeRestriction);

        SmApiResult result = policyapi.addPolicy(policy, true);

        if (!result.isSuccess())
        {
            return result;
        }

        SmRule rule = new SmRule();
        result = policyapi.getRule(OBJNAME_PREFIX + "grule",
                                   SmObjId.NULL_OID.toString(),
                                   SmObjId.NULL_OID.toString(),
                                   rule);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_GET_RULE"));
        }

        SmPolicyLink policyLink = new SmPolicyLink();
        policyLink.setPolicy(policy.getOid());
        policyLink.setRule(rule.getOid());

        result = policyapi.addPolicyLink(policyLink, true);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_POLICY_LINK"));
        }

        // set Response to the policy link
        SmResponse response = new SmResponse();
        result = policyapi.getResponse(OBJNAME_PREFIX + "gresponse",
                                       SmObjId.NULL_OID.toString(),
                                       response);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_GET_RESPONSE"));
        }

        policyLink.setResponse(response.getOid());
        result = policyapi.setResponseInPolicyLink(policyLink);

        if (!result.isSuccess())
        {
            LogError(bundle.getString("FAILED_SET_RESPONSE"));
        }
        return result;
    }

    private SmApiResult
    createVariable(SmPolicyApi policyapi)
        throws SmApiException
    {
        SmVariable variable = new SmVariable();
        variable.setName("Variable");
        variable.setReturnType(SmVariable.TYPE_STRING);

        SmVariableType variableType = new SmVariableType();
        policyapi.getVariableType(SmVariableType.USER_CONTEXT_VARIABLES, variableType);

        SmDomain domain = new SmDomain();
        policyapi.getDomain(OBJNAME_PREFIX + "domain", domain);

        variable.setDomain(domain);
        variable.setVariableType(variableType);
        variable.setDescription("Variable");
        variable.setPrefetchFlag(true);
        SmApiResult result = policyapi.addVariable(variable);
        return result;
    }

    private int
    resetPolicyStore(SmPolicyApi policyapi)
    {
        int status = 0;
        SmApiResult result = null;
        Log(CRLF + bundle.getString("INIT_POLICY_STORE"));

        try
        {
            policyapi.deleteAdmin(OBJNAME_PREFIX + "admin");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteGroup(SmGroup.Agent_Group_Prop,
                                  OBJNAME_PREFIX + "agentgroup",
                                  "");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteDomain(OBJNAME_PREFIX + "domain");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteAgentConfig(OBJNAME_PREFIX + "agentconfig");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteHostConfig(OBJNAME_PREFIX + "hostconfig");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            result = policyapi.deleteTrustedHost(OBJNAME_PREFIX + "trustedhost");
            File ConfigFile = new File(bundle.getString("HOSTCONFIGFILE"));
            ConfigFile.delete();
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }
        try
        {
            Vector globalNames = new Vector();
            result = policyapi.getDomainObjectNames(SmDomain.PropRules,
                                                    SmObjId.NULL_OID.toString(), 
                                                    globalNames);
            if (result.isSuccess())
            {
                for (int i = 0; i < globalNames.size(); ++i)
                {
                    SmRule rule = new SmRule((String) globalNames.get(i));
                    result = policyapi.deleteRule(rule);
                }
            }
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            Vector globalNames = new Vector();
            result = policyapi.getDomainObjectNames(SmDomain.PropResponses,
                                                    SmObjId.NULL_OID.toString(), 
                                                    globalNames);
            if (result.isSuccess())
            {
                for (int i = 0; i < globalNames.size(); ++i)
                {
                    SmResponse response = new SmResponse((String) globalNames.get(i));
                    result = policyapi.deleteResponse(response);
                }
            }
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteAgent(OBJNAME_PREFIX + "agent");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteScheme(OBJNAME_PREFIX + "authscheme");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteSelfReg(OBJNAME_PREFIX + "regscheme");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deletePasswordPolicy(OBJNAME_PREFIX + "pwdpolicy");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            Vector globalNames = new Vector();
            result = policyapi.getGlobalObjectNames(SmCertMap.PropCertMaps,
                                                    globalNames);

            if (result.isSuccess())
            {
                for (int i = 0; i < globalNames.size(); ++i)
                {
                    SmObjId certMapId = SmObjId.valueOf((String) globalNames.get(i));
                    result = policyapi.deleteCertMap(certMapId);
                }
            }
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            Vector globalNames = new Vector();
            result = policyapi.getDomainObjectNames(SmDomain.PropPolicies,
                                                    SmObjId.NULL_OID.toString(), 
                                                    globalNames);

            if (result.isSuccess())
            {
                for (int i = 0; i < globalNames.size(); ++i)
                {
                    SmPolicy policy = new SmPolicy((String) globalNames.get(i));
                    result = policyapi.deletePolicy(policy);
                }
            }
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            Vector globalNames = new Vector();
            result = policyapi.getGlobalObjectNames(SmAuthAzMap.PropAuthAzMaps,
                                                    globalNames);

            if (result.isSuccess())
            {
                for (int i = 0; i < globalNames.size(); ++i)
                {
                    SmObjId authAzMapId = SmObjId.valueOf((String) globalNames.get(i));
                    result = policyapi.deleteAuthAzMap(authAzMapId);
                }
            }
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteODBCQuery(OBJNAME_PREFIX + "queryscheme");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }

        try
        {
            policyapi.deleteUserDirectory(OBJNAME_PREFIX + "userdir");
        }
        catch (SmApiException sme)
        {
            LogException(sme);
            status = -1;
        }
        Log(STATUS_OK);
        return status;
    }


    private static void
    Log(String log)
    {
        System.out.print(log);

        if (logWriter != null)
        {
            logWriter.print(log);
        }
    }


    private static void
    LogError(String errLog)
    {
        Log(CRLF + "ERROR: " + errLog);
    }


    private static void
    LogException(Exception e)
    {
        Log(CRLF + "EXCEPTION: " + e.getMessage());
    }


    private static void
    LogFatal(String fatalLog)
    {
        Log(CRLF + "FATAL: " + fatalLog);

        if (logWriter != null)
        {
            logWriter.close();
        }

        System.exit(-1);
    }
}

// EOF PolicyApiSample.java
